#define NEWCI_G

#include <stdio.h>
#include <ctype.h>

/* UNIX specific */
#include <sys/types.h> 
#include <sys/wait.h> 

#include <time.h>

#include <utils/p2c.h>
#include <graphics/newci.h>
#include <graphics/newcrt.h>
#include <utils/strings.h>
#include <utils/newasm.h>
#include <utils/sysdevs.h>


/*  This procedure should be made to do something real. <-- I agree. */
void newci_inputstring(char *s)
{
	char buf[1024];
	char *cp;

	nc_fgets(buf, 1024, stdin);
	if ((cp = (char *) strchr(buf, '\n')))
		*cp = '\0';
	if (*buf)
		strcpy(s, buf);
}


static void goodfixname(char *name, char *ext, char *home, int force)
{
	size_t i;
	int hassuffix;
	char STR1[256];
	size_t len = strlen(name);

	/* fs_zapspaces(name); */
	if (len > 2 && name[0] == '*' && name[1] == '*')
	{
		strcpy_overlap(name, name + 2);

		sprintf(name, "%s%s", home, strcpy(STR1, name));
	}

	if (force)
	{
		i = strlen(name);
		while (i > 0 && name[i - 1] != '.' && name[i - 1] != '/' && name[i - 1] != ':')
			i--;
		if (i > 0 && name[i - 1] == '.')
			name[i - 1] = '\0';
	}

	if (*name == '\0')
		return;

	i = strlen(name) - 1;
	if (name[i] == '.')
	{
		name[i] = '\0';
		return;
	}

	if (name[i] == ':')
		return;
	
	hassuffix = false;
	
	len = strlen(name);
	for (i = 0; i < len; ++i)
	{
		switch (name[i])
		{

			case '/':
			case ':':
			hassuffix = false;
			break;

			case '.':
			hassuffix = true;
			break;
		}
	}

	if (!hassuffix && *ext != '\0')
	{
		/* Disabling reallocation because all strings are char[256], will change in future */
		/* Warning: when doing reallocation, return char* or take char** because of reallocation */
		//name = realloc(name, sizeof(char) * (len + strlen(ext) + 2)); /* name + '.' + ext + '\0' */
		sprintf(name + strlen(name), ".%s", ext);
	}
}

void newci_fixfname(char * s, char * ext, char * home)
{
	goodfixname(s, ext, home, 0);
}

void newci_forcefname(char *s, char *ext, char *home)
{
	goodfixname(s, ext, home, 1);
}

void newci_fullshellescape(char *args, char *opts)
{
	while (isspace(*args) || *args == '!')
		args++;

	if (fork())
		wait(NULL);
	else
		if (*args)
			execlp("/bin/csh", "xterm", "-e", args, (char *)0);
		else
			execlp("/bin/csh", "xterm", (char *)0);
}

void newci_shellescape(char *args)
{
	newci_fullshellescape(args, "");
}

void newci_fulleditescape(char *fn, char *opts, long ln, long pos)
{
	if (fork())
		wait(NULL);
	else
		execlp("emacs", "emacs", fn, (char *)0);
}


void newci_eatargument()
{
	int i;

	/* move the argmuments to the left*/
	for (i = 2; i < P_argc; ++i)
		P_argv[i-1] = P_argv[i];
	P_argc--;
}

void newci_parseswitch(newci_parserec* tab, size_t size, char *bad)
{
	int found;
	size_t pos = 1;

	for (size_t i = 0; i < size; ++i)
		tab[i].used = 0;
	*bad = '\000';
  
	/* still args and arg start with '-' */
	while (P_argc > 1 && P_argv[1][0] == '-')
	{
		found = 0;
		size_t i;
		for (i = 0; i < size && !found; i++)
		{
    		for (size_t j = 0; tab[i].switch_[j] != '\0' && !found; j++)
			{
				if (tab[i].switch_[j] == P_argv[1][1]) 
					found = 1;
			}
		}
		i--; /* reset i to found index */

		if (!found)
		{
    		bad[strlen(bad) + 1] = '\0';
    		bad[strlen(bad)] = P_argv[1][1];
		}
		else
		{
			tab[i].used = pos++;
			
			switch (toupper(tab[i].kind))
			{
				case 'B': /* boolean */
				break;
				case 'I': /* int */
				tab[i].UU.i = atol(P_argv[1]+2);
				break;
				case 'R': /* real */
				sscanf(P_argv[1]+2, "%lf", &tab[i].UU.r);
				break;
				case 'S': /* string */
				tab[i].UU.s = P_argv[1]+2;
				break;
				case 'M': /* multiple */
				if (P_argc > 2 && !P_argv[1][2])
				{
					tab[i].UU.s = P_argv[2];
					newci_eatargument();
				}
				else
				{
					tab[i].UU.s = P_argv[1] + 2;
				}
				break;
			}
		}
		newci_eatargument();
	}
}

long timers_sysclock()
{
	return sysclock();
}

long newci_fullseconds()
{
	return(time(NULL));
}

void newci_nullrecover()
{
	_Escape(P_escapecode);
}
